//+----------------------------------------------------------+
//|                              Ehlers fisher transform.mq4 |
//|                                                   mladen |
//+----------------------------------------------------------+
#property  copyright "mladen"
#property  link      "mladenfx@gmail.com"

#property  indicator_separate_window
#property  indicator_buffers 4
#property  indicator_color1  clrDeepSkyBlue
#property  indicator_color2  clrSandyBrown
#property  indicator_color3  clrSandyBrown
#property  indicator_color4  clrSilver
#property  indicator_width1  3
#property  indicator_width2  3
#property  indicator_width3  3
#property  indicator_style4  STYLE_DOT
#property  strict

//
//
//
//
//

enum enFilterWhat
{
   flt_prc,  // Filter the price
   flt_val,  // Filter the fisher transform value
   flt_both  // Filter both
};

extern int                period       = 10;             // Transform period
extern ENUM_APPLIED_PRICE PriceType    = PRICE_MEDIAN;   // Price to use
extern double             Weight       = 2;              // Smoothing weight
extern double             Filter       = 0;              // Filter to use for filtering (<=0 - no filtering)
extern int                FilterPeriod = 0;              // Filter period (0<= to use transform period)
extern enFilterWhat       FilterOn     = flt_val;        // Apply filter to :
extern bool               alertsOn        = true;        // Turn alerts on?
extern bool               alertsOnCurrent = false;       // Alerts on current (still opened) bar?
extern bool               alertsMessage   = true;        // Alerts should show pop-up message?
extern bool               alertsSound     = false;       // Alerts should play alert sound?
extern bool               alertsPushNotif = false;       // Alerts should send push notification?
extern bool               alertsEmail     = false;       // Alerts should send email?

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];
double buffer4[];
double Prices[];
double Values[];
double Cross[];

   
//+----------------------------------------------------------+
//|                                                          |
//+----------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
      SetIndexBuffer(0,buffer1);
      SetIndexBuffer(1,buffer2);
      SetIndexBuffer(2,buffer3);
      SetIndexBuffer(3,buffer4);
      SetIndexBuffer(4,Prices);
      SetIndexBuffer(5,Values);
      SetIndexBuffer(6,Cross);
   IndicatorShortName("Ehlers\' Fisher transform ("+(string)period+","+(string)Filter+")");
   return(0);
}


//----------------------------------------------------------
//
//----------------------------------------------------------
//
//
//
//
//

int start()
{
   static int previousCounted=0; int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

   int    tperiod = FilterPeriod; if (tperiod<=0) tperiod = period;
   double pfilter = Filter;       if (FilterOn==flt_val) pfilter=0;
   double vfilter = Filter;       if (FilterOn==flt_prc) vfilter=0;
   double alpha = 2.0/(1.0+Weight);         
   if (Cross[limit]==-1) CleanPoint(limit,buffer2,buffer3);
   for(int i=limit; i>=0; i--)
   {  
      Prices[i] = iFilter(iMA(NULL,0,1,0,MODE_SMA,PriceType,i),pfilter,tperiod,i,0);
      
      //
      //
      //
      //
      //
                  
         double MaxH = Prices[ArrayMaximum(Prices,period,i)];
         double MinL = Prices[ArrayMinimum(Prices,period,i)];
         if (MaxH!=MinL && i<Bars-1)
               Values[i] = alpha*((Prices[i]-MinL)/(MaxH-MinL)-0.5+Values[i+1]);
         else  Values[i] = 0.00;
               Values[i] = MathMin(MathMax(Values[i],-0.999),0.999); 

      // 
      //
      //
      //
      //

      if (i<Bars-1)
      {
         buffer1[i] = iFilter(0.5*MathLog((1+Values[i])/(1-Values[i]))+0.5*buffer1[i+1],vfilter,tperiod,i,1);
         buffer2[i] = EMPTY_VALUE;
         buffer3[i] = EMPTY_VALUE;
         buffer4[i] = buffer1[i+1];
         Cross[i]   = Cross[i+1];
            if (buffer1[i]>buffer4[i]) Cross[i]=  1;
            if (buffer1[i]<buffer4[i]) Cross[i]= -1;
            if (Cross[i]==-1) PlotPoint(i,buffer2,buffer3,buffer1);
      }
      else buffer1[i] = 0;               
   }

   //
   //
   //
   //
   //
   
      if (alertsOn)
      {
        int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
        if (Cross[whichBar] != Cross[whichBar+1])
        {
           if (Cross[whichBar] ==  1) doAlert(whichBar,"up");
           if (Cross[whichBar] == -1) doAlert(whichBar,"down");
        }
      }        
   return(0);
}


//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Ahlers fisher transform trend changed to "+doWhat;
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(Symbol()+" Ahlers fisher transform",message);
          if (alertsPushNotif) SendNotification(message);
          if (alertsSound)     PlaySound("alert2.wav");
   }
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

#define filterInstances 2
double workFil[][filterInstances*3];

#define _fchange 0
#define _fachang 1
#define _fprice  2

double iFilter(double tprice, double filter, int tperiod, int i, int instanceNo=0)
{
   if (filter<=0) return(tprice);
   if (ArrayRange(workFil,0)!= Bars) ArrayResize(workFil,Bars); i = Bars-i-1; instanceNo*=3;
   
   //
   //
   //
   //
   //
   
   workFil[i][instanceNo+_fprice]  = tprice; if (i<1) return(tprice);
   workFil[i][instanceNo+_fchange] = MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]);
   workFil[i][instanceNo+_fachang] = workFil[i][instanceNo+_fchange];

   for (int k=1; k<tperiod && (i-k)>=0; k++) workFil[i][instanceNo+_fachang] += workFil[i-k][instanceNo+_fchange];
                                            workFil[i][instanceNo+_fachang] /= tperiod;
    
   double stddev = 0; for (int k=0;  k<tperiod && (i-k)>=0; k++) stddev += MathPow(workFil[i-k][instanceNo+_fchange]-workFil[i-k][instanceNo+_fachang],2);
          stddev = MathSqrt(stddev/(double)tperiod); 
   double filtev = filter * stddev;
   if( MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]) < filtev ) workFil[i][instanceNo+_fprice]=workFil[i-1][instanceNo+_fprice];
        return(workFil[i][instanceNo+_fprice]);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}