//+------------------------------------------------------------------+
//|                                    HilbertSineWave_v1.0 600+.mq4 |
//|                                Copyright  2016, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2016, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"
#property link      "http://newdigital-world.com"


#property indicator_separate_window
#property indicator_buffers 2

#property indicator_color1 clrTomato
#property indicator_color2 clrDeepSkyBlue

#property indicator_width1 2
#property indicator_width2 1


enum ENUM_PRICE
{
   close,               // Close
   open,                // Open
   high,                // High
   low,                 // Low
   median,              // Median
   typical,             // Typical
   weightedClose,       // Weighted Close
   heikenAshiClose,     // Heiken Ashi Close
   heikenAshiOpen,      // Heiken Ashi Open
   heikenAshiHigh,      // Heiken Ashi High   
   heikenAshiLow,       // Heiken Ashi Low
   heikenAshiMedian,    // Heiken Ashi Median
   heikenAshiTypical,   // Heiken Ashi Typical
   heikenAshiWeighted   // Heiken Ashi Weighted Close   
};


#define pi 3.14159265358979323846
//---- indicator parameters

input ENUM_TIMEFRAMES   TimeFrame               =        0;       // TimeFrame
input ENUM_PRICE        Price                   =        4;       // Apply to 
input double            Alpha                   =     0.07;       // Cycle Smoothing Factor
input int               MedianPeriod            =        5;       // Period of Moving Median
input int               DCsmooth                =        5;       // Period of Dominant Cycle Smoothing
input double            AdvanceAngle            =       90;       // Advance Angle for lag compensation
input double            LeadSineAngle           =       45;       // Leading Sine Angle
input bool              ShowSupportResistance   =        0;       // Show Support and Resistance
input color             SupportColor            = clrDeepSkyBlue;
input color             ResistanceColor         = clrOrangeRed;
input int               LineWidth               =        2;
input int               LineStyle               =        0;
input string            UniqueName              = "SineWave";    

//---- indicator buffers
double sine[];
double lead[];
double cycle[];

//----
int      timeframe, draw_begin, trend[2];
string   TF, IndicatorName, short_name, price_name;
datetime prevtime[8], prevrstime, sTime[2], rTime[2];
double   support[2], resistance[2];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   timeframe = TimeFrame;
   if(timeframe <= Period()) timeframe = Period(); 
   TF = tf(timeframe);
   
   IndicatorDigits(Digits);
//---- indicator buffers mapping
   IndicatorBuffers(3);
   SetIndexBuffer(0, sine); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1, lead); SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(2,cycle); 
   
//---- indicator name
       
   IndicatorName = WindowExpertName(); 
   short_name = IndicatorName + "[" + TF + "]("+ EnumToString(Price) + "," + DoubleToStr(Alpha,3) + "," + DCsmooth +")";
   IndicatorShortName(short_name);
      
   SetIndexLabel(0,"Sine");
   SetIndexLabel(1,"Lead");
   
//----
   draw_begin = Bars - iBars(NULL,timeframe)*timeframe/Period() + DCsmooth + MedianPeriod;
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);


//---- initialization done
 
   return(0);
}


//----
int deinit()
{
   deleteObj(UniqueName);
   
   return(0);
}
//+------------------------------------------------------------------+
//| HilbertSineWave_v1.0 600+                                        |
//+------------------------------------------------------------------+
int start()
{
   int limit, i, shift, counted_bars = IndicatorCounted(); 
   
   
   if(counted_bars > 0) limit=Bars-counted_bars-1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
   { 
   limit = Bars-1;   
      for(i=limit;i>=0;i--) 
      {
      sine[i] = EMPTY_VALUE; 
      lead[i] = EMPTY_VALUE; 
      }
   }   
  
   if(timeframe != Period())
	{
   limit = MathMax(limit,timeframe/Period());   
      
      for(shift=0;shift<limit;shift++) 
      {	
      int y = iBarShift(NULL,timeframe,Time[shift]);
            
      sine[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Alpha,MedianPeriod,DCsmooth,AdvanceAngle,LeadSineAngle,ShowSupportResistance,SupportColor,ResistanceColor,LineWidth,LineStyle,UniqueName,0,y);      
      lead[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Alpha,MedianPeriod,DCsmooth,AdvanceAngle,LeadSineAngle,ShowSupportResistance,SupportColor,ResistanceColor,LineWidth,LineStyle,UniqueName,1,y);      
               
      }  
	
	return(0);
	}
	else _SineWave(limit);

   return(0);
}

//-----

void _SineWave(int limit)
{   
   
         
   for(int shift=limit;shift>=0;shift--) 
   {
      if(prevrstime != Time[shift])
      {
      support[1]    = support[0];   
      resistance[1] = resistance[0];
      trend[1]      = trend[0];
      sTime[1]      = sTime[0];
      rTime[1]      = rTime[0];
      prevrstime    = Time[shift];
      }
   double period = cyclePeriod(Price,Alpha,MedianPeriod,DCsmooth,shift);
   int Length = MathFloor(period);      
   
   if(Length == 0) continue;
    
   //Compute Dominant Cycle Phase

   double RealPart = 0, ImagPart = 0;
   
      for(int i=0;i<Length;i++)
      {
      RealPart += MathSin(2*pi*i/Length)*cycle[shift+i];
      ImagPart += MathCos(2*pi*i/Length)*cycle[shift+i];
      }

   if(MathAbs(ImagPart) > 0.00000001) double DcPhase = 180/pi*MathArctan(RealPart/ImagPart); 
   else {if(RealPart > 0) DcPhase = 90; else DcPhase = -90;}
   
   DcPhase += AdvanceAngle;
   
   if(ImagPart <   0) DcPhase += 180;
   if(DcPhase  > 315) DcPhase -= 360;
   
   sine[shift] = MathSin(DcPhase*pi/180);
   lead[shift] = MathSin((DcPhase + LeadSineAngle)*pi/180);
   
      if(ShowSupportResistance)
      {
      trend[0] = trend[1];
      sTime[0] = sTime[1];
      rTime[0] = rTime[1];
      
      if(lead[shift] > sine[shift] && lead[shift+1] <= sine[shift+1]) trend[0] = 1;
      if(lead[shift] < sine[shift] && lead[shift+1] >= sine[shift+1]) trend[0] =-1;
        
         if(trend[0] > 0)
         {
         support[0] = support[1];
         
            if(trend[0] != trend[1])
            {
            support[0] = Low[shift];
            sTime[0]   = Time[shift];
            }
         
         if(sTime[0] > 0) PlotLine(UniqueName + "_sup_" + TimeToStr(sTime[0]),0,sTime[0],support[0],Time[shift],support[0],SupportColor,LineWidth,LineStyle);
         }               
            
         if(trend[0] < 0)
         {
         resistance[0] = resistance[1];
            
            if(trend[0] != trend[1])
            {
            resistance[0] = High[shift];
            rTime[0]      = Time[shift];
            }
         
         if(rTime[0] > 0) PlotLine(UniqueName + "_res_" + TimeToStr(rTime[0]),0,rTime[0],resistance[0],Time[shift],resistance[0],ResistanceColor,LineWidth,LineStyle);
         }               
      }    
   }
}
   
//-----
double   DeltaPhase[], aPrice[4], smooth[3], Q1[2], I1[2], iPeriod[2], cPeriod[4];
datetime prevcptime;

double cyclePeriod(int price,double alpha,int median,int dcsmooth,int bar)
{      
   if(ArraySize(DeltaPhase) != Bars) 
   {
   ArraySetAsSeries(DeltaPhase,false); 
   ArrayResize(DeltaPhase,Bars); 
   ArraySetAsSeries(DeltaPhase,true); 
   }   
      
   if(prevcptime != Time[bar])
   {
      for(int i=3;i>=1;i--) 
      {
      aPrice[i]  = aPrice[i-1];
      cPeriod[i] = cPeriod[i-1]; 
      if(i < 3) smooth[i] = smooth[i-1];
      }
   
   I1[1] = I1[0];
   Q1[1] = Q1[0];
   iPeriod[1] = iPeriod[0];
   prevcptime = Time[bar];
   }   

      if(price <= 6) aPrice[0] = iMA(NULL,0,1,0,0,(int)price,bar);   
      else
      if(price > 6 && price <= 13) aPrice[0] = HeikenAshi(price-7,Bars,bar);
   
         
   if(bar < Bars - 3) smooth[0] = (aPrice[0] + 2*aPrice[1] + 2*aPrice[2] + aPrice[3])/6; 
	cycle[bar] = (1 - 0.5*alpha)*(1 - 0.5*alpha)*(smooth[0] - 2*smooth[1] + smooth[2]) + 2*(1-alpha)*cycle[bar+1] - (1-alpha)*(1-alpha)*cycle[bar+2];
	if(bar > Bars - 7) cycle[bar] = (aPrice[0] - 2*aPrice[1] + aPrice[2])/4;  
	
		   
   Q1[0] = (0.0962*cycle[bar] + 0.5769*cycle[bar+2] - 0.5769*cycle[bar+4] - 0.0962*cycle[bar+6])*(0.5+0.08*iPeriod[1]);
   I1[0] = cycle[bar+3];  
	
   if(Q1[0] != 0 && Q1[1] != 0) DeltaPhase[bar] = (I1[0]/Q1[0] - I1[1]/Q1[1])/(1 + I1[0]*I1[1]/(Q1[0]*Q1[1]));		
   if(DeltaPhase[bar] < 0.1) DeltaPhase[bar] = 0.1;  
   if(DeltaPhase[bar] > 1.1) DeltaPhase[bar] = 1.1;
   double MedianDelta = MedianOnArray(DeltaPhase,median,bar);

   if(MedianDelta == 0) double DC = 15; else DC = 6.28318/MedianDelta + 0.5;
	
   iPeriod[0] = 0.33*DC + (1 - 2.0/(dcsmooth + 1))*iPeriod[1];
   cPeriod[0] = 0.15*iPeriod[0] + 0.85*cPeriod[1];
   
   if(cPeriod[3] == 0) return(0);   
     
   return(cPeriod[0]);
}

double MedianOnArray(double& price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i=0;i<per;i++) array[i] = price[bar+i];
   ArraySort(array,WHOLE_ARRAY,0,MODE_DESCEND);
   
   int num = MathRound((per - 1)*0.5); 
   if(MathMod(per,2) > 0) double median = array[num]; else median = 0.5*(array[num] + array[num+1]);
    
   return(median); 
}
   
void PlotLine(string name,int win,datetime time1,double value1,datetime time2,double value2,color clr,int width,int style)
{
   ObjectDelete(name);
   bool res = ObjectCreate(name,OBJ_TREND,win,time1,value1,time2,value2);
   ObjectSet(name, OBJPROP_WIDTH,width);
   ObjectSet(name, OBJPROP_STYLE,style);
   ObjectSet(name, OBJPROP_RAY  ,false);
   ObjectSet(name, OBJPROP_BACK ,false);
   ObjectSet(name, OBJPROP_COLOR,clr  );
}        

bool deleteObj(string name)
{
   bool result = false;
   
   int length = StringLen(name);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
   string objName = ObjectName(i); 
   if(StringSubstr(objName,0,length) == name) {ObjectDelete(objName); result = true;}
   }
   
   return(result);
}

// HeikenAshi Price:  7 - Close,8 - Open,9 - High,10 - Low 
double   haClose[2], haOpen[2], haHigh[2], haLow[2];
datetime prevhatime;

double HeikenAshi(int price,int cbars,int bar)
{ 
   if(prevhatime != Time[bar])
   {
   haClose[1] = haClose[0];
   haOpen [1] = haOpen [0];
   haHigh [1] = haHigh [0];
   haLow  [1] = haLow  [0];
   prevhatime = Time[bar];
   }
   
   if(bar == cbars - 1) 
   {
   haClose[0] = Close[bar];
   haOpen [0] = Open [bar];
   haHigh [0] = High [bar];
   haLow  [0] = Low  [bar];
   }
   else
   {
   haClose[0] = (Open[bar] + High[bar] + Low[bar] + Close[bar])/4;
   haOpen [0] = (haOpen[1] + haClose[1])/2;
   haHigh [0] = MathMax(High[bar],MathMax(haOpen[0],haClose[0]));
   haLow  [0] = MathMin(Low [bar],MathMin(haOpen[0],haClose[0]));
   }
   
   switch(price)
   {
   case  0: return(haClose[0]); break;
   case  1: return(haOpen [0]); break;
   case  2: return(haHigh [0]); break;
   case  3: return(haLow  [0]); break;
   case  4: return((haHigh[0] + haLow[0])/2); break;
   case  5: return((haHigh[0] + haLow[0] +   haClose[0])/3); break;
   case  6: return((haHigh[0] + haLow[0] + 2*haClose[0])/4); break;
   default: return(haClose[0]); break;
   }
} 

string tf(int itimeframe)
{
   string result = "";
   
   switch(itimeframe)
   {
   case PERIOD_M1:   result = "M1" ;
   case PERIOD_M5:   result = "M5" ;
   case PERIOD_M15:  result = "M15";
   case PERIOD_M30:  result = "M30";
   case PERIOD_H1:   result = "H1" ;
   case PERIOD_H4:   result = "H4" ;
   case PERIOD_D1:   result = "D1" ;
   case PERIOD_W1:   result = "W1" ;
   case PERIOD_MN1:  result = "MN1";
   default:          result = "N/A";
   }
   
   if(result == "N/A")
   {
   if(itimeframe <  PERIOD_H1 ) result = "M"  + itimeframe;
   if(itimeframe >= PERIOD_H1 ) result = "H"  + itimeframe/PERIOD_H1;
   if(itimeframe >= PERIOD_D1 ) result = "D"  + itimeframe/PERIOD_D1;
   if(itimeframe >= PERIOD_W1 ) result = "W"  + itimeframe/PERIOD_W1;
   if(itimeframe >= PERIOD_MN1) result = "MN" + itimeframe/PERIOD_MN1;
   }
   
   return(result); 
}
  







